import Automator
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestAMAction(TestCase):
    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsPrintf(Automator.AMAction.logMessageWithLevel_format_, 1)
        self.assertArgIsOut(Automator.AMAction.runWithInput_error_, 1)

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertArgIsOut(Automator.AMAction.initWithContentsOfURL_error_, 1)
        self.assertArgIsBOOL(Automator.AMAction.initWithDefinition_fromArchive_, 1)
        self.assertResultIsBOOL(Automator.AMAction.ignoresInput)

    def testMethods(self):
        self.assertArgIsOut(Automator.AMAction.runWithInput_fromAction_error_, 2)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertResultIsBOOL(Automator.AMAction.isStopped)

    def testConstants(self):
        self.assertEqual(Automator.AMLogLevelDebug, 0)
        self.assertEqual(Automator.AMLogLevelInfo, 1)
        self.assertEqual(Automator.AMLogLevelWarn, 2)
        self.assertEqual(Automator.AMLogLevelError, 3)



"""
Script for building the example.

Usage:
    python3 setup.py py2app

Then install the bundle in dist into ~/Library/Automator.
"""
from setuptools import setup

infoPlist = {
    "AMAccepts": {
        "Container": "List",
        "Optional": False,
        "Types": ["com.apple.addressbook.person-object"],
    },
    "AMApplication": ["Address Book", "iChat"],
    "AMCanShowWhenRun": True,
    "AMCategory": "iChat",
    "AMDefaultParameters": {},
    "AMDescription": {
        "AMDAlert": "iChat must be running for this action to work properly.",
        "AMDNote": "Information will not be returned for the current user.",
        "AMDSummary": "This action returns the Instant Message information "
        "of the people passed from the previous action.",
    },
    "AMIconName": "iChat",
    "AMKeywords": ("Instant", "Message", "IM"),
    "AMName": "Get Buddy Info",
    "AMProvides": {"Container": "List", "Types": ["com.apple.cocoa.string"]},
}

setup(
    name="Get Buddy Info",
    plugin=["GetBuddyInfo.py"],
    data_files=[],
    options={"py2app": {"extension": ".action", "plist": infoPlist}},
    setup_requires=[
        "py2app",
        "pyobjc-framework-Automator",
        "pyobjc-framework-AddressBook",
        "pyobjc-framework-InstantMessage",
        "pyobjc-framework-Cocoa",
    ],
)



import Automator
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestAMWorkspace(TestCase):
    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsOut(Automator.AMWorkspace.runWorkflowAtPath_withInput_error_, 2)



# This file is generated by objective.metadata
#
# Last update: Tue Mar  3 21:32:20 2020
#
# flake8: noqa

import objc, sys

if sys.maxsize > 2 ** 32:

    def sel32or64(a, b):
        return b


else:

    def sel32or64(a, b):
        return a


misc = {}
constants = """$$"""
enums = """$AMActionApplicationResourceError@-206$AMActionApplicationVersionResourceError@-207$AMActionArchitectureMismatchError@-202$AMActionExceptionError@-213$AMActionExecutionError@-212$AMActionFailedGatekeeperError@-217$AMActionFileResourceError@-208$AMActionInitializationError@-211$AMActionInsufficientDataError@-215$AMActionIsDeprecatedError@-216$AMActionLicenseResourceError@-209$AMActionLinkError@-205$AMActionLoadError@-204$AMActionMalwareError@-221$AMActionNotLoadableError@-201$AMActionPropertyListInvalidError@-214$AMActionQuarantineError@-219$AMActionRequiredActionResourceError@-210$AMActionRuntimeMismatchError@-203$AMActionSignatureCorruptError@-218$AMActionThirdPartyActionsNotAllowedError@-222$AMActionXProtectError@-220$AMConversionFailedError@-302$AMConversionNoDataError@-301$AMConversionNotPossibleError@-300$AMLogLevelDebug@0$AMLogLevelError@3$AMLogLevelInfo@1$AMLogLevelWarn@2$AMNoSuchActionError@-200$AMUserCanceledError@-128$AMWorkflowActionsNotLoadedError@-113$AMWorkflowNewerActionVersionError@-111$AMWorkflowNewerVersionError@-100$AMWorkflowNoEnabledActionsError@-114$AMWorkflowOlderActionVersionError@-112$AMWorkflowPropertyListInvalidError@-101$"""
misc.update(
    {
        "AMAutomatorErrorDomain": "com.apple.Automator",
        "AMActionErrorKey": "AMActionErrorKey",
    }
)
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(b"AMAction", b"ignoresInput", {"retval": {"type": "Z"}})
    r(
        b"AMAction",
        b"initWithContentsOfURL:error:",
        {"arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"AMAction",
        b"initWithDefinition:fromArchive:",
        {"arguments": {3: {"type": "Z"}}},
    )
    r(b"AMAction", b"isStopped", {"retval": {"type": "Z"}})
    r(
        b"AMAction",
        b"logMessageWithLevel:format:",
        {"arguments": {3: {"printf_format": True}}, "variadic": True},
    )
    r(b"AMAction", b"runWithInput:error:", {"arguments": {3: {"type_modifier": b"o"}}})
    r(
        b"AMAction",
        b"runWithInput:fromAction:error:",
        {"arguments": {4: {"type_modifier": b"o"}}},
    )
    r(b"AMBundleAction", b"hasView", {"retval": {"type": "Z"}})
    r(
        b"AMBundleAction",
        b"initWithDefinition:fromArchive:",
        {"arguments": {3: {"type": "Z"}}},
    )
    r(b"AMShellScriptAction", b"remapLineEndings", {"retval": {"type": "Z"}})
    r(
        b"AMWorkflow",
        b"initWithContentsOfURL:error:",
        {"arguments": {3: {"type_modifier": b"o"}}},
    )
    r(
        b"AMWorkflow",
        b"runWorkflowAtURL:withInput:error:",
        {"arguments": {4: {"type_modifier": b"o"}}},
    )
    r(b"AMWorkflow", b"setValue:forVariableWithName:", {"retval": {"type": "Z"}})
    r(
        b"AMWorkflow",
        b"writeToURL:error:",
        {"retval": {"type": "Z"}, "arguments": {3: {"type_modifier": b"o"}}},
    )
    r(b"AMWorkflowController", b"canRun", {"retval": {"type": "Z"}})
    r(b"AMWorkflowController", b"isPaused", {"retval": {"type": "Z"}})
    r(b"AMWorkflowController", b"isRunning", {"retval": {"type": "Z"}})
    r(b"AMWorkflowView", b"isEditable", {"retval": {"type": "Z"}})
    r(b"AMWorkflowView", b"setEditable:", {"arguments": {2: {"type": "Z"}}})
    r(
        b"AMWorkspace",
        b"runWorkflowAtPath:withInput:error:",
        {"arguments": {4: {"type_modifier": b"o"}}},
    )
    r(
        b"NSObject",
        b"workflowController:didError:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"workflowController:didRunAction:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"workflowController:willRunAction:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"workflowControllerDidRun:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"workflowControllerDidStop:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"workflowControllerWillRun:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"workflowControllerWillStop:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
finally:
    objc._updatingMetadata(False)
protocols = {
    "AMWorkflowControllerDelegate": objc.informal_protocol(
        "AMWorkflowControllerDelegate",
        [
            objc.selector(
                None, b"workflowControllerDidStop:", b"v@:@", isRequired=False
            ),
            objc.selector(
                None, b"workflowControllerWillRun:", b"v@:@", isRequired=False
            ),
            objc.selector(
                None, b"workflowController:willRunAction:", b"v@:@@", isRequired=False
            ),
            objc.selector(
                None, b"workflowControllerDidRun:", b"v@:@", isRequired=False
            ),
            objc.selector(
                None, b"workflowController:didRunAction:", b"v@:@@", isRequired=False
            ),
            objc.selector(
                None, b"workflowControllerWillStop:", b"v@:@", isRequired=False
            ),
            objc.selector(
                None, b"workflowController:didError:", b"v@:@@", isRequired=False
            ),
        ],
    )
}
expressions = {}

# END OF FILE



import struct
from AddressBook import ABAddressBook, ABPerson
from Automator import AMBundleAction
from Cocoa import NSAppleEventDescriptor
from InstantMessage import (
    IMPersonFirstNameKey,
    IMPersonLastNameKey,
    IMPersonScreenNameKey,
    IMPersonServiceNameKey,
    IMPersonStatusAvailable,
    IMPersonStatusAway,
    IMPersonStatusIdle,
    IMPersonStatusKey,
    IMPersonStatusMessageKey,
    IMPersonStatusOffline,
    IMPersonStatusUnknown,
    IMService,
)

typeObjectSpecifier = struct.unpack(">i", b"obj ")[0]
keyAEContainer = struct.unpack(">i", b"from")[0]
keyAEKeyData = struct.unpack(">i", b"seld")[0]


class GetBuddyInfo(AMBundleAction):
    def runWithInput_fromAction_error_(self, value, anAction, errorInfo):
        people = []

        # convert the input to a list of ABPerson objects
        if isinstance(value, NSAppleEventDescriptor):
            count = value.numberOfItems()

            for i in range(1, count + 1):
                personDescriptor = value.descriptorAtIndex_(i)
                if personDescriptor is not None:
                    # get the uid of the person from this descriptor
                    if personDescriptor.descriptorType() == typeObjectSpecifier:
                        container = personDescriptor.descriptorForKeyword_(
                            keyAEContainer
                        )
                        if container is not None:
                            uidDescriptor = container.descriptorAtIndex_(
                                i
                            ).descriptorForKeyword_(keyAEKeyData)
                            if uidDescriptor is not None:
                                uid = uidDescriptor.stringValue()
                                if uid is not None:
                                    # get the person object from the uid
                                    person = ABAddressBook.sharedAddressBook().recordForUniqueId_(  # noqa: B950
                                        uid
                                    )
                                    if person is not None:
                                        people.append(person)

        info = []
        for person in people:
            for service in IMService.allServices():
                if isinstance(person, ABPerson):
                    screenNames = service.screenNamesForPerson_(person)
                    if screenNames is not None:
                        for screenName in screenNames:
                            info_dict = service.infoForScreenName_(screenName)
                            if info_dict is not None:
                                # build the description
                                description = "\rName: "

                                firstName = info_dict[IMPersonFirstNameKey]
                                if firstName is not None:
                                    description += firstName
                                    description += " "

                                lastName = info_dict[IMPersonLastNameKey]
                                if lastName is not None:
                                    description += lastName

                                description += "\r"

                                serviceName = info_dict[IMPersonServiceNameKey]
                                if serviceName is not None:
                                    description += "Service: "
                                    description += serviceName
                                    description += "\r"

                                screenName = info_dict[IMPersonScreenNameKey]
                                if screenName is not None:
                                    description += "Screen Name: "
                                    description += screenName
                                    description += "\r"

                                status = info_dict[IMPersonStatusKey]
                                if status is not None:
                                    description += "Status: "

                                    if status == IMPersonStatusUnknown:
                                        description += "Unknown"
                                    elif status == IMPersonStatusOffline:
                                        description += "Offline"
                                    elif status == IMPersonStatusIdle:
                                        description += "Idle"
                                    elif status == IMPersonStatusAway:
                                        description += "Away"
                                    elif status == IMPersonStatusAvailable:
                                        description += "Available"

                                    description += "\r"

                                message = info_dict[IMPersonStatusMessageKey]
                                if message is not None:
                                    description += "Message: "
                                    description += message
                                    description += "\r"

                                info.append(description)
        return str(info)
